# -*- coding: utf-8 -*-

# This file is part of the Gedit LaTeX Plugin
#
# Copyright (C) 2007 Michael Zeising
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public Licence as published by the Free Software
# Foundation; either version 2 of the Licence, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public Licence for more 
# details.
#
# You should have received a copy of the GNU General Public Licence along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA  02110-1301, USA

from gtk.gdk import keyval_name
from gobject import timeout_add, source_remove
from re import compile

from LaTeXPlugin.Logger import debug, error
from CommandStore import CommandStore, Choice
from CommandsParser import CommandsParser
from PrefixParser import PrefixParser
from LaTeXPlugin.common import ConditionalSource
from LaTeXPlugin.Installation import Installation
from LaTeXPlugin.util import str_bool

from LaTeXPlugin.bibtex.BibTeXEntry import BibTeXEntry
from LaTeXPlugin.latex.LatexParser import BibItem


class Controller(object):
	"""
	The main controller for autocompletion
	"""
	
	MAX_PREFIX_LENGTH = 50

	ABORT_KEYS = [ "Escape", "Left", "Right", "Home", "End" ]
	NAVIGATION_KEYS = [ "Up", "Down", "Page_Up", "Page_Down" ]
	
	__pattern_begin = compile(r"\\begin\{(?P<name>[a-zA-Z]+\*?)\}?$")
	
	def __init__(self, wrapper, textview, popup, hint_popup, settings):
		self.__wrapper = wrapper
		self.__textview = textview
		self.__textbuffer = textview.get_buffer()
		
		self.__popup = popup
		self.__hint_popup = hint_popup
		
		# read settings
		self.__trigger_keys = []
		if str_bool(settings.setdefault("TriggerBackslash", True)):
			self.__trigger_keys.append("backslash")
			
		if str_bool(settings.setdefault("TriggerBraceLeft", True)):
			self.__trigger_keys.append("braceleft")
			
		if str_bool(settings.setdefault("TriggerBracketLeft", True)):
			self.__trigger_keys.append("bracketleft")
			
		if str_bool(settings.setdefault("TriggerParenLeft", True)):
			self.__trigger_keys.append("parenleft")
		
		self.__enable_prompt = str_bool(settings.setdefault("PromptCompletion", True))
		self.__delay = int(settings.setdefault("CompletionDelay", 500))
		self.__enable_delayed = str_bool(settings.setdefault("DelayedCompletion", True))
		
		# connect to signals
		self.__textview.connect("key-press-event", self.__key_pressed)
		self.__textview.connect_after("key-release-event", self.__key_released)
		self.__textview.connect("button-press-event", lambda *x: self.__stop())
		self.__textview.connect("focus-out-event", lambda *x: self.__stop())
		
		# TODO: parser may be a singleton
		debug(self, "__init__", "Loading LaTeX definition...")
		#self.__command_store = CommandStore("%s/autocomplete/definition.xml" % install_path)
		
		#self.__command_store = CommandStore()
		from CommandStore import CommandStoreFactory
		
		self.__command_store = CommandStoreFactory().getCommandStore()
		
		#p = CommandsParser()
		#p.parse(self.__command_store, "%s/autocomplete/commands.xml" % install_path)
		
		debug(self, "__init__", "Loading LaTeX definition... done")
		
		self.__parser = PrefixParser(self.__command_store)
		
		self.__newcommand_names = []
		
		self.__ctrl_pressed = False
		self.__timer = None
	
	def set_newcommands(self, newcommands):
		debug(self, "set_newcommands", str([c.name for c in newcommands]))
		
		# remove old commands
		for name in self.__newcommand_names:
			del self.__command_store[name]
		self.__newcommand_names = []
			
		# store new ones
		for newcommand in newcommands:
			self.__newcommand_names.append(newcommand.name)
			self.__command_store[newcommand.name] = newcommand
	
	def set_labels(self, labels):
		choices = [Choice(label) for label in labels]
		self.__command_store.fill_placeholder("Labels", choices)
	
	def set_open_environment(self, environment):
		"""
		The currently open environment is attached as a choice of the \end command
		"""
		self.__command_store.fill_placeholder("OpenEnvironment", [Choice(environment)])
	
	def set_bibitems(self, bibitems):
		"""
		Load BibTeX entries of type bibtex.BibTeXParser.Entry
		"""
		choices = [Choice(x.key, None, x.markup) for x in bibitems if type(x) is BibTeXEntry or type(x) is BibItem]
		self.__command_store.fill_placeholder("Bibitems", choices)
		
		
	#
	# TODO: combine those methods
	# 
	def set_ambient_tex_files(self, files):
		choices = [Choice(f) for f in files]
		self.__command_store.fill_placeholder("TexFiles", choices)
	
	def set_ambient_image_files(self, files):
		choices = [Choice(f) for f in files]
		self.__command_store.fill_placeholder("ImageFiles", choices)
		
	def set_ambient_bib_files(self, files):
		choices = [Choice(f) for f in files]
		self.__command_store.fill_placeholder("BibFiles", choices)
	
	
	
	def __key_pressed(self, textview, event):
		key_value = keyval_name(event.keyval)
		
		if key_value in self.ABORT_KEYS:
			self.__stop()

		#
		# recognize triggering key events
		#
		self._prompt = False
		if self.__ctrl_pressed:
			if self.__enable_prompt and key_value == "space":
				self._prompt = True
				self.__trigger()
				return False
		
		#
		# trigger
		#
		if key_value == "Control_L" or key_value == "Control_R":
			self.__ctrl_pressed = True
		else:
			self.__ctrl_pressed = False
		
		
		if self.__enable_delayed and key_value in self.__trigger_keys:
			self.__trigger_delayed()
			return False
		
		#
		# pass navigation events to the popup if it is active
		#
		if key_value in self.NAVIGATION_KEYS and self.__popup.get_active():
			self.__popup.navigate(key_value)
			return True
		
		#
		# accept a proposal
		#
		if key_value == "Return" and self.__popup.get_active():
			self.__insert_proposal(self.__popup.selected_proposal)
			self.__stop()
			self.__auto_close(env_selected=True)
			return True
		
		if self.__timer:
			source_remove(self.__timer)
		
		#
		# stop ac on every other key
		#
		if not self.__popup.get_active():
			self.__stop()
	
	def __key_released(self, textview, event):
		key_value = keyval_name(event.keyval)
		
		# auto close on }
		if key_value == "braceright":
			self.__auto_close(brace_typed=True)
		
		if self.__popup.get_active():
			if key_value in self.NAVIGATION_KEYS:
				# don't propagate navigation keys
				return True
			
			# type on after auto completion has been triggered
			self.__trigger()
	
	def __insert_proposal(self, proposal):
		"""
		Insert a proposal into the TextBuffer by completing
		"""
		left, right = proposal.pair
		n = self.count_overlapping_chars(self.__parser.subprefix, left)
		left = left[n:]
		
		if right:
			source = (left, right)
		else:
			source = left
		
		if proposal.package:		
			source = ConditionalSource(source, [proposal.package])
		
		self.__wrapper.insert_source(source)
	
	def __trigger(self):
		"""
		Try to auto complete immediatelly
		"""
		prefix = self.__recognize_prefix()
		
		if prefix:
			self.__parser.parse(prefix)
			
			if len(self.__parser.proposals) == 1 and not self.__popup.get_active() and self._prompt:
				debug(self, "__trigger", "Only one proposal, popup inactive -> auto inserting...")
				self.__insert_proposal(self.__parser.proposals[0])
				self.__stop()
			elif len(self.__parser.proposals) == 0:
				self.__stop()
			else:
				# n > 1 or ( n == 1 and popup.get_active() )
				self.__popup.set_proposals(self.__parser.proposals)
				self.__popup.move_to_cursor(self.__textview)
				self.__popup.set_active(True)
			
			if self.__parser.hint:
				self.__hint_popup.set_hint(self.__parser.hint)
				self.__hint_popup.move_to_cursor(self.__textview)
				self.__hint_popup.set_active(True)
		else:
			self.__stop()
	
	def __trigger_delayed(self, timeout=False):
		"""
		Start a short timer for auto complete 
		"""
		debug(self, "__trigger_delayed", "timeout=%s" % timeout)
		
		if timeout:
			self.__trigger()
			return False	# do not call again
		else:
			if self.__timer:
				source_remove(self.__timer)
			self.__timer = timeout_add(self.__delay, self.__trigger_delayed, True)
			
	def __stop(self):
		"""
		Stop autocompletion
		"""
		self.__popup.set_active(False)
		self.__hint_popup.set_active(False)
	
	def __recognize_prefix(self):
		"""
		Try to find a TeX prefix left of the cursor
		"""
		it_insert = self.__textbuffer.get_iter_at_mark(self.__textbuffer.get_insert())
		it_search = it_insert.copy()
		
		i = 0
		while i < self.MAX_PREFIX_LENGTH:
			if not it_search.backward_char():
				return None
			
			c = it_search.get_char()
			if c in ["\\", " ", "\t", "\n"]:
				break
			
			i += 1
		
		if i == self.MAX_PREFIX_LENGTH:
			debug(self, "__recognize_prefix", "Prefix is too long")
			return None
		
		# check for \\ and don't complete there
		it_t = it_search.copy()
		if it_t.backward_char():
			if it_t.get_char() == "\\":
				debug(self, "__recognize_prefix", "not completing at \\\\")
				return None
		
		prefix = self.__textbuffer.get_text(it_search, it_insert, False)
		debug(self, "__recognize_prefix", prefix)
		
		return prefix
	
	def __auto_close(self, brace_typed=False, env_selected=False):
		"""
		Try to auto close
		"""
		prefix = self.__recognize_prefix()
		
		if prefix:
			match = self.__pattern_begin.match(prefix)
			
			if match:
				name = match.group("name")
				
				if brace_typed:
					# \begin{abc}
					#           ^
					self.__wrapper.insert_source(("", "\\end{%s}" % name))
				elif env_selected:
					s = self.__wrapper.get_text_after_cursor(1)
					if s == "}":
						# \begin{abc}
						#          ^
						self.__wrapper.move_cursor(1)
						self.__wrapper.insert_source(("", "\\end{%s}" % name))
					else:
						# \begin{abc
						#          ^
						self.__wrapper.insert_source(("}", "\\end{%s}" % name))
	
	@staticmethod
	def count_overlapping_chars(a, b):
		# TODO: isn't there a built-in function for this?
		n = 0
		for i in range(min(len(a), len(b))):
			if a[i] == b[i]:
				n += 1
			else:
				break
		debug(None, "count_overlapping_chars", "'%s', '%s' : %s common chars" % (a, b, n))
		return n 
		