# -*- coding: utf-8 -*-

# This file is part of the Gedit LaTeX Plugin
#
# Copyright (C) 2007 Michael Zeising
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public Licence as published by the Free Software
# Foundation; either version 2 of the Licence, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public Licence for more 
# details.
#
# You should have received a copy of the GNU General Public Licence along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA  02110-1301, USA

class Token(object):
	UNDEFINED, TOP_LEVEL_COMMENT, AT, EOF = -1, 0, 1, 2
	
	def __init__(self, code, content, line, column):
		self.code = code
		self.content = content
		self.line = line
		self.column = column
		
	def __str__(self):
		return "%s: '%s'" % (self.choice, self.content)


class ParseException(Exception):
	def __init__(self, line, column, encountered, expected):
		self.line = line
		self.column = column
		self.encountered = encountered
		self.expected = expected


class BibTeXLexer(object):
	def __init__(self, input):
		self.__input = input
		self.__eof_token = None
	
	@property
	def reader(self):
		return self.__input
	
	def scan_top_level(self):
		"""
		Scan for a comment, an @ character or EOF and return the according Token
		"""
		self.skip_whitespace()
		
		if self.__eof_token:
			return Token(Token.EOF, self.__eof_token.content, self.__eof_token.line, 
						self.__eof_token.column)
		
		column = self.__input.column
		line = self.__input.line
		
		if self.__input.current == "@":
			self.__input.step()
			return Token(Token.AT, "@", line, column)
		
		content = []
		
		while not self.__input.eof and self.__input.current != "@":
			content.append(self.__input.current)
			self.__input.step()
		
		return Token(Token.TOP_LEVEL_COMMENT, "".join(content), line, column)
	
	def scan_alternatives(self, alternatives, look_ahead):
		"""
		The return value is an index into alternatives. If lookAhead is true we
		will not move forward ...
		"""
		self.skip_whitespace()
		
		if self.__eof_token:
			raise ParseException(self.__eof_token.line, self.__eof_token.column, "[EOF]", \
								self.__alternatives_to_string(alternatives))
		
		column = self.__input.column
		line = self.__input.line
		
		for i in range(len(alternatives)):
			if alternatives[i] == self.__input.current:
				if not look_ahead:
					self.__input.step()
				return i
		
		if not look_ahead:
			self.__input.step()
		
		raise ParseException(line, column, "" + self.__input.current, self.__alternatives_to_string(alternatives))
	
	def scan_literal(self, termination_set, exclude_whitespace, enforce_nonzero):
		if exclude_whitespace:
			self.skip_whitespace()
			
		if self.__eof_token:
			raise ParseException(self.__eof_token.line, self.__eof_token.column, "[EOF]", \
								"not (" + self.__alternatives_to_string(termination_set) + " or [whitespace])")
		
		self.enforce_no_eof("not (" + self.__alternatives_to_string(termination_set) + ")", False)
		
		column = self.__input.column
		line = self.__input.line
		
		buffer = ""
		
		while True:
			if self.__input.eof:
				break
			
			input_char = self.__input.current
			
			if input_char in termination_set or exclude_whitespace and input_char.isspace():
				break
			else:
				self.__input.step()
				buffer += input_char
		
		if len(buffer) > 0 or not enforce_nonzero:
			return buffer
		else:
			raise ParseException(line, column, "" + self.__input.current, 
								"not (" + self.__alternatives_to_string(termination_set) + " or [whitespace])")
	
	def __scan_literal(self, termination_set, exclude_whitespace, enforce_nonzero, buffer):
		"""
		The return value is an index into the termination set the result is
		appended in the target_buffer
		
		Returns (choice, buf)
		"""
		
		if exclude_whitespace:
			self.skip_whitespace()
			
		if self.__eof_token:
			raise ParseException(self.__eof_token.line, self.__eof_token.column, "[EOF]", \
								"not (" + self.__alternatives_to_string(termination_set) + " or [whitespace])")
		
		self.enforce_no_eof("not (" + self.__alternatives_to_string(termination_set) + ")", False)
		
		column = self.__input.column
		line = self.__input.line
		
		index_into_termination_set = -1
		initial_target_buffer_length = len(buffer)
		
		while True:
			if self.__input.eof:
				break
			
			input_char = self.__input.current
			
			# TODO: use real try...except here
			try:
				index_into_termination_set = termination_set.index(input_char)
			except ValueError:
				index_into_termination_set = -1
			
			if index_into_termination_set >= 0 or exclude_whitespace and input_char.isspace():
				break
			else:
				self.__input.step()
				buffer += input_char
		
		if len(buffer) > initial_target_buffer_length or not enforce_nonzero:
			return (index_into_termination_set, buffer)
		else:
			raise ParseException(line, column, "" + self.__input.current, 
								"not (" + self.__alternatives_to_string(termination_set) + " or [whitespace])")
	
	QUOTE_OR_LBRACE = ['\"', '{']
	
	def scan_quoted_string(self):
		buffer = ""
		
		self.scan('"')
		
		while True:
			index, buffer = self.__scan_literal(self.QUOTE_OR_LBRACE, False, False, buffer)
			
			if index == 0:		# we terminated with '"'
				break
			else:				# we found a '{'
				buffer = self.scan_bracketed_string(buffer, True)
			
		self.scan('"')
		
		return buffer

	RBRACE_LBRACE = ['}', '{']
	
	def scan_bracketed_string(self, buffer, include_outer_braces):
		"""
		Returns new buffer
		"""
		_buffer = buffer
		
		self.scan('{')
		
		if include_outer_braces:
			_buffer += "{"
		
		while True:
			index, _buffer = self.__scan_literal(self.RBRACE_LBRACE, False, False, _buffer)
			
			if index == 0:		# we terminated with '}'
				break
			else:
				_buffer = self.scan_bracketed_string(_buffer, True)
			
		self.scan('}')
		
		if include_outer_braces:
			_buffer += '}'
		
		return _buffer
	
	def scan_entry_type_name(self):
		self.skip_whitespace()
		
		if self.__eof_token:
			raise ParseException(self.__eof_token.line, self.__eof_token.column, "[EOF]", "[a..z,A..Z]")
		
		column = self.__input.column
		line = self.__input.line
		
		result = []
		
		while True:
			# Check for EOF
			#self.enforce_no_eof("[a..z,A..Z]", False)
			if self.__input.eof:
				self.__eof_token = Token(Token.UNDEFINED, None, self.__input.line, self.__input.column)
				raise ParseException(self.__eof_token.line, self.__eof_token.column, "[EOF]", "[a..z,A..Z]")
				
			input_char = self.__input.current
			
			if input_char.isalpha():
				result.append(input_char)
				self.__input.step()
			else:
				break
		
		if len(result) == 0:
			raise ParseException(line, column, "" + self.__input.current, "[a..z,A..Z]")
		
		return "".join(result)
	
	def scan(self, expected):
		self.skip_whitespace()
		
		if self.__eof_token:
			raise ParseException(self.__eof_token.line, self.__eof_token.column, "[EOF]", "" + expected)
		
		encountered = self.__input.current
		
		if encountered != expected:
			column = self.__input.column
			line = self.__input.line
			self.__input.step()
			raise ParseException(line, column, expected, encountered)
		else:
			self.__input.step()
			
	def skip_whitespace(self):
		if self.__eof_token:
			return
		
		while not self.__input.eof and self.__input.current.isspace():
			self.__input.step()
			
		if self.__input.eof:
			self.__eof_token = Token(Token.UNDEFINED, None, self.__input.line, self.__input.column)
	
	def enforce_no_eof(self, expected, skip_whitespace):
		if skip_whitespace:
			self.skip_whitespace()
		elif self.__input.eof:
			self.__eof_token = Token(Token.UNDEFINED, None, self.__input.line, self.__input.column)
		
		if self.__eof_token:
			raise ParseException(self.__eof_token.line, self.__eof_token.column, "[EOF]", "" + expected)
	
	def current_input_char(self):
		return self.__input.current
	
	@staticmethod
	def __alternatives_to_string(alternatives):
		buffer = "one of "
		for i in range(len(alternatives)):
			if i > 0:
				buffer += ","
			buffer += "'" + alternatives[i] + "'"
		return buffer
	