# -*- coding: utf-8 -*-

# This file is part of the Gedit LaTeX Plugin
#
# Copyright (C) 2007 Michael Zeising
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public Licence as published by the Free Software
# Foundation; either version 2 of the Licence, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public Licence for more 
# details.
#
# You should have received a copy of the GNU General Public Licence along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Sdocumentt, Fifth Floor, Boston, MA  02110-1301, USA

from gobject import idle_add

from LaTeXPlugin.Logger import debug

from BibTeXLexer import BibTeXLexer, Token, ParseException
from BibTeXToplevelComment import BibTeXToplevelComment
from BibTeXMultipleValues import BibTeXMultipleValues
from BibTeXPreamble import BibTeXPreamble
from BibTeXString import BibTeXString
from BibTeXEntry import BibTeXEntry
from BibTeXModel import BibTeXModel
from StringReader import StringReader

class BibTeXParser(object):
	# TODO: implement parsing as really threaded, maybe see gobject.threads_init
	
	KEEP_FIRST, KEEP_LAST, KEEP_ALL = 0, 1, 2
	
	def __init__(self):
		self.__throw_all_parse_exceptions = False
		self.__lexer = None
		self.__model = None
		self.__exceptions = []
		self.__multiple_field_values_policy = self.KEEP_FIRST
	
	def get_exceptions(self):
		return self.__exceptions
	
	def parse(self, model, reader, idle=True):
		assert isinstance(model, BibTeXModel)
		assert isinstance(reader, StringReader)
		
		self.__lexer = BibTeXLexer(reader)
		self.__model = model
		
		if idle:
			idle_add(self.__parse)
		else:
			self.__parse()
		
	def __parse(self):
		self.__exceptions = []
		
		while True:
			token = self.__lexer.scan_top_level()
			
			if token.code == Token.TOP_LEVEL_COMMENT:
				self.__model.add_entry(BibTeXToplevelComment(self.__lexer.reader.line, 
															self.__lexer.reader.column, token.content))
				
			elif token.code == Token.AT:
				if self.__throw_all_parse_exceptions:
					self.__parse_entry()
				else:
					try:
						self.__parse_entry()
					except ParseException, e:
						self.__exceptions.append(e)
						
			elif token.code == Token.EOF:
				return
	
	EXCEPTION_SET_NAMES = ['"', '#', '%', '\'', '(', ')', ',', '=', '{', '}']
	
	def __parse_entry(self):
		lexer = self.__lexer
		model = self.__model
		
		entry_type = lexer.scan_entry_type_name().lower()
		
		bracket_choice = lexer.scan_alternatives(['{', '('], False)
		
		if entry_type == "string":
			string_name = lexer.scan_literal(self.EXCEPTION_SET_NAMES, True, True)
			lexer.scan("=")
			value = self.__parse_value()
			
			assert string_name != None
			assert value != None
			assert not type(value) is BibTeXMultipleValues
			
			model.add_entry(BibTeXMacroDefinition(self.__lexer.reader.line, self.__lexer.reader.column, 
												string_name, value))
			
		elif entry_type == "preamble":
			value = self.__parse_value()
			
			assert value != None
		
			model.add_entry(BibTeXPreamble(self.__lexer.reader.line, self.__lexer.reader.column, value))
			
		else:
			lexer.skip_whitespace()
			
			if lexer.current_input_char() == ",":
				entry_key = ""
			else:
				entry_key = lexer.scan_literal([","], True, True)
			
			assert entry_type != None
		
			if not entry_key:
				entry_key = ""
			
			entry = BibTeXEntry(self.__lexer.reader.line - 1, self.__lexer.reader.column, entry_type, entry_key)
			
			while True:
				lexer.enforce_no_eof("',' or corresponding closing bracket", True)
	
				if lexer.current_input_char() == ",":
					lexer.scan(",")
					lexer.enforce_no_eof("'}' or [FIELDNAME]", True)
					
					if lexer.current_input_char() == "}":
						break
					
					field_name = lexer.scan_literal(self.EXCEPTION_SET_NAMES, True, True)
					lexer.scan("=")
					
					value = self.__parse_value()
					
					if self.__multiple_field_values_policy == self.KEEP_FIRST:
						try:
							entry.get_field_value(field_name)
						except KeyError:
							entry.set_field(field_name, value)
					elif self.__multiple_field_values_policy == self.KEEP_LAST:
						entry.set_field(field_name, value)
					elif self.__multiple_field_values_policy == self.KEEP_ALL:
						entry.add_field_value(field_name, value)
				else:
					break
			
			model.add_entry(entry)

		if bracket_choice == 0:
			lexer.scan("}")
		else:
			lexer.scan(")")
			
	def __parse_value(self):
		self.__lexer.enforce_no_eof("[STRING] or [STRINGREFERENCE] or [NUMBER]", True)
		input_character = self.__lexer.current_input_char()
		
		result = None
		if input_character == '"':		# quoted string...
			buf = self.__lexer.scan_quoted_string()
			assert buf != None
			result = BibTeXString(self.__lexer.reader.line, self.__lexer.reader.column, buf)
			
		elif input_character == "{":	# bracketed string...
			buf = self.__lexer.scan_bracketed_string("", False)
			assert buf != None
			result = BibTeXString(self.__lexer.reader.line, self.__lexer.reader.column, buf)
			
		else:
			string_content = self.__lexer.scan_literal(self.EXCEPTION_SET_NAMES, False, True).strip()
			if string_content.isdigit():
				assert string_content != None
				result = BibTeXString(self.__lexer.reader.line, self.__lexer.reader.column, string_content)
			else:
				assert string_content != None
				result = BibTeXMacroReference(self.__lexer.reader.line, self.__lexer.reader.column, 
											string_content)
		
		self.__lexer.enforce_no_eof("'#' or something else", True)
		
		if self.__lexer.current_input_char() == "#":
			self.__lexer.scan("#")
			
			right = self.__parse_value()
			assert right != None
			
			return BibTeXConcatenatedValue(self.__lexer.reader.line, self.__lexer.reader.column, result, 
										right)
		else:
			return result
		
		
		
if __name__ == "__main__":
	from sys import argv
	from FileReader import FileReader
	from BibTeXTree import BibTeXTree
	
	t = BibTeXTree()
	p = BibTeXParser()
	p.parse(t, FileReader(argv[1]), idle=False)
	
	
		