from gtk import RESPONSE_OK, STOCK_CONVERT, STOCK_INFO

from BuildEngine import BuildEngine
from LaTeXPlugin.Logger import error, debug
from LaTeXPlugin.document.DocumentHelper import LANGUAGE_LATEX
from BuildView import BuildView
from LaTeXPlugin.ConsoleView import ConsoleView

class BuildController(object):
	def __init__(self, window_helper):
		self.__window_helper = window_helper
		
		# create engine
		self.__engine = BuildEngine()
		
		self.__engine.connect("job-started", self.__job_started)
		self.__engine.connect("job-aborted", self.__job_aborted)
		self.__engine.connect("job-failed", self.__job_failed)
		self.__engine.connect("job-finished", self.__job_finished)
		self.__engine.connect("output", self.__output)
		
		# create ui elements
		self.__build_view = BuildView()
		
		self.__build_view.connect("line-selected", self.__line_selected)
		self.__build_view.connect("abort-clicked", lambda x: self.__engine.abort())
		
		self.__console_view = ConsoleView()
		
		self.__window_helper.window.get_bottom_panel().add_item(self.__build_view, "Build", 
															STOCK_CONVERT)
		self.__window_helper.window.get_bottom_panel().add_item(self.__console_view, "Console Output", 
															STOCK_INFO)
		
	def build(self, profile):
		helper = self.__window_helper.document_helper_manager.active_helper
		
		if not helper:
			error(self, "build", "No active wrapper")
			return
		
		self.__window_helper.save_active_document()
		
		if not helper.filename:
			return
		
		if helper.get_language() != LANGUAGE_LATEX:
			# TODO: popup error message
			error(self, "build", "no LaTeX")
			return
		
		latex_helper = helper.language_helper
		latex_helper.reset_highlight()
		
		filename = None
		if latex_helper.is_master:
			filename = helper.filename
		else:
			# try to get master from meta file
			meta_file = helper.meta_file
			if meta_file != None:
				try:
					filename = meta_file["TexMasterFilename"]
					debug(self, "build", "Master found in meta file: %s" % filename)
				except KeyError:
					debug(self, "build", "Master NOT found in meta file")
			else:
				print meta_file
				debug(self, "build", "No meta file")
			
			# ask the user
			if not filename:
				from LaTeXPlugin.MasterDialog import MasterDialog
				
				dialog = MasterDialog(self.__window_helper.window)
				
				f = latex_helper.master_filename
				if f:
					dialog.set_filename(latex_helper.master_filename)	# guess and proposal
				
				if dialog.run() == RESPONSE_OK:
					filename = dialog.get_filename()
					# store it to meta file
					meta_file["TexMasterFilename"] = filename
					meta_file.save()
					
				dialog.destroy()
		
		if not filename:
			return
		
		self.__engine.set_filename(filename)
		self.__engine.set_profile(profile)
		
		self.__build_view.set_profile(profile)
		
		self.__engine.run()
	
	def deactivate(self):
		self.__window_helper.window.get_bottom_panel().remove_item(self.__build_view)
		self.__window_helper.window.get_bottom_panel().remove_item(self.__console_view)
	
	def __activate_view(self):
		"""
		Activate the build view
		"""
		chk = self.__window_helper.window.get_ui_manager().get_widget("/MenuBar/ViewMenu/ViewBottomPaneMenu")
		if chk:
			chk.set_active(True)
		
		self.__window_helper.window.get_bottom_panel().activate_item(self.__build_view)
	
	def __line_selected(self, widget, filename, line):
		"""
		"""
		debug(self, "__line_selected", "%s, %s : %s" % (type(widget), filename, line))
		
		try:
			helper = self.__window_helper.document_helper_manager.activate_helper(filename)
			helper.jump_to_line(line)
		except KeyError:
			error(self, "__line_selected", "Failed")
	
	def __job_started(self, engine, job_id):
		self.__build_view.set_job_started(job_id)
	
	def __job_aborted(self, engine, job_id):
		self.__build_view.set_job_aborted(job_id)
		
	def __job_failed(self, engine, job_id, summary, messages):
		self.__build_view.set_job_failed(job_id)
		self.__build_view.set_summary(job_id, summary)
		self.__build_view.append_messages(job_id, messages)
		
		self.__process_messages(messages)
		
		self.__activate_view()
		
	def __job_finished(self, engine, job_id, summary, messages):
		self.__build_view.set_job_finished(job_id)
		self.__build_view.set_summary(job_id, summary)
		self.__build_view.append_messages(job_id, messages)
		
		self.__process_messages(messages)
	
	def __process_messages(self, messages):
		for message in messages:
			try:
				#wrapper = self.__instance.get_wrapper_by_filename(message.filename)
				helper = self.__window_helper.document_helper_manager.find_helper(message.filename)
				latex_helper = helper.language_helper
				
				if message.line_from:
					if message.line_to:
						# TODO: wrapper.highlight_lines(lines)
						for line in range(int(message.line_from), int(message.line_to) + 1):
							latex_helper.highlight_line(line, message.severity)
					else:
						latex_helper.highlight_line(message.line_from, message.severity)
			except KeyError:
				error(self, "__process_messages", "Could not select the right document for highlighting")
	
	def __output(self, engine, output, type):
		self.__console_view.append(output, bool(type == BuildEngine.T_COMMAND))
		