# -*- coding: utf-8 -*-

# This file is part of the Gedit LaTeX Plugin
#
# Copyright (C) 2007 Michael Zeising
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public Licence as published by the Free Software
# Foundation; either version 2 of the Licence, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public Licence for more 
# details.
#
# You should have received a copy of the GNU General Public Licence along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA  02110-1301, USA

class GridBuilder(object):
	"""
	This used to build the content grid of a table either by reading
	data from a file or by creating placeholders
	"""
	def __init__(self):
		self.n_rows = 0
		self.n_columns = 0
		self.grid = None
	
	def load_from_file(self, filename):
		try:
			values = [row.split() for row in open(filename).readlines() if row[0] != '#' and len(row.split())]
			self.n_rows = len(values)
			if self.n_rows:
				self.n_columns = len(values[0])
			else:
				self.n_columns = 0
					
			if not self.n_rows or not self.n_columns:
				raise Exception("File does not contain tabular data.")
			
			rows = ["\t" + " & ".join(row) for row in values]
			self.grid = " \\\\\n".join(rows)
		except IOError, s:
			raise Exception(s)
	
	def create_placeholders(self, n_rows, n_columns):
		r = "\t" + " & ".join("x" * n_columns)
		self.grid = " \\\\\n".join([r] * n_rows)


from LaTeXPlugin.common import ConditionalSource
from LaTeXPlugin.sourcebuilder import * 


class TableSourceBuilder(object):
	"""
	This is used to generate the TeX source for a tabular object 
	"""
	def __init__(self):
		self.grid = None
		
		self.n_rows = 0
		self.n_columns = 0
		
		self.context_text = True
		self.column_style = None
		
		self.context_math = False
		self.math_environment = "matrix"
		
		self.float = False
		self.caption = None
		self.label = None
	
	def get_source(self):
		if self.context_text:
			tabular = Environment("tabular")
			tabular.arguments.append(MandatoryArgument(self.column_style))
			tabular.append(self.grid)
			
			center = Environment("center")
			center.append(tabular)
			
			if self.float:
				# floating / text context
				table = Environment("table")
				table.append(center)
				
				if self.caption:
					captionof = Command("caption")
					captionof.arguments.append(MandatoryArgument(self.caption))
					center.append(captionof)
				
				if self.label:
					label = Command("label")
					label.arguments.append(MandatoryArgument(self.label))
					center.append(label)
				
				return str(table)
			else:
				# not floating / text context
				if self.caption:
					captionof = Command("captionof")
					captionof.arguments.append(MandatoryArgument("table"))
					captionof.arguments.append(MandatoryArgument(self.caption))
					
					center.append(captionof)
				
				if self.label:
					label = Command("label")
					label.arguments.append(MandatoryArgument(self.label))
					
					center.append(label)
				
				return ConditionalSource(str(center), ["caption"])
		else:
			# math context
			env = Environment(self.math_environment)
			env.append(self.grid)
			
			return ConditionalSource(str(env), ["amsmath"])
	